# VecGeomConfig.cmake
# -------------------
#
# Config mode file for the VecGeom includes and libraries
#
# IMPORTED Targets
# ^^^^^^^^^^^^^^^^
#
# This module defines the following `IMPORTED` targets:
#
# - VecGeom::vecgeom            - VecGeom library
# - VecGeom::vgdml              - Present if the GDML parser component was built
# - VecGeom::vecgeomcuda_static - Present if the CUDA backend was built.
#                                 Static library containing all cuda-related parts of VecGeom with
#                                 device code that can be linked/device-linked against if client code 
#                                 uses VecGeom device functions/kernels. It automatically provides the 
#                                 dependency to the vecgeom library.
# - VecGeom::vecgeomcuda        - Present if the CUDA backend was built
#                                 Shared library with same objects as VecGeom::vecgeomcuda_static and
#                                 including a device link object. Non-CUDA using clients can link against
#                                 this directly. CUDA using clients should prefer the static version.
#
# Result Variables
# ^^^^^^^^^^^^^^^^
#
# This module defines the following variables either directly, or automatically
# though CMake's find_package command.
#
# - VecGeom_FOUND                 - True if VecGeom was found.
# - VecGeom_INCLUDE_DIRS          - Location of the VecGeom header files.
# - VecGeom_LIBRARIES             - The VecGeom libraries
#                                   Use of the ``IMPORTED`` targets described above should be preferred.
# - VecGeom_shared_FOUND          - True if the VecGeom C++ library is shared.
# - VecGeom_static_FOUND          - True if the VecGeom C++ library is static.
# - VecGeom_GDML_FOUND            - True if the VecGeom GDML parser library is available.
# - VecGeom_single_precision_FOUND - True if using single-precision floats
# - VecGeom_CUDA_FOUND            - True if VecGeom was built with CUDA support.
# - VecGeom_scalar_backend_FOUND  - True if the VecGeom backend is scalar.
# - VecGeom_vector_backend_FOUND  - True if the VecGeom backend is vector.
# - VecGeom_vc_backend_FOUND      - True if the VecGeom vector backend is Vc.
#
# - VecGeom_BUILTIN_VECCORE       - True if VecGeom was built with an internal version on VecCore.
# - VecGeom_CXX_STANDARD          - ISO C++ Standard used to compile VecGeom (e.g. "17").
# - VecGeom_VECTOR_ISA            - String holding the Instruction Set Architecture VecGeom
#                                   was compiled against (e.g. "sse4.2"). The empty string
#                                   if no explicit flag was used.
# - VecGeom_CUDA_STANDARD         - CUDA/C++ Standard used to compile VecGeom's CUDA library (e.g. "17")
#                                   Unset unless ``VecGeom_CUDA_FOUND`` is ``TRUE``.
# - VecGeom_CUDA_ARCHITECTURES    - Value of ``CMAKE_CUDA_ARCHITECTURES`` used to compile/link
#                                   VecGeom's CUDA library. Unset unless ``VecGeom_CUDA_FOUND`` is ``TRUE``.
#
# The ``VecGeom_<component>_FOUND`` variables provide support for component arguments 
# to ``find_package``, allowing consumers to require that the found VecGeom supports the component
# i.e.
#
# .. code-block:: cmake
#
#   find_package(VecGeom REQUIRED <component1> ... <componentN>)
#


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was VecGeomConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../../" ABSOLUTE)

# Use original install prefix when loaded through a "/usr move"
# cross-prefix symbolic link such as /lib -> /usr/lib.
get_filename_component(_realCurr "${CMAKE_CURRENT_LIST_DIR}" REALPATH)
get_filename_component(_realOrig "/usr/lib/arm-linux-gnueabihf/cmake/VecGeom" REALPATH)
if(_realCurr STREQUAL _realOrig)
  set(PACKAGE_PREFIX_DIR "/usr")
endif()
unset(_realOrig)
unset(_realCurr)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

# Basic variables
set_and_check(VECGEOM_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/include")
set(VECGEOM_INCLUDE_DIRS ${VECGEOM_INCLUDE_DIR})

# Full version string
set(VecGeom_VERSION_STRING "1.2.8")

# Configuration options
set(VecGeom_BACKEND "Scalar")
set(VecGeom_BUILTIN_VECCORE "OFF")
set(VecGeom_DISTANCE_DEBUG "OFF")
set(VecGeom_EMBREE "OFF")
set(VecGeom_ENABLE_CUDA "OFF")
set(VecGeom_ERROR_LIMIT "20")
set(VecGeom_FAST_MATH "OFF")
set(VecGeom_GDML "ON")
set(VecGeom_GDMLDEBUG "OFF")
set(VecGeom_INPLACE_TRANSFORMATIONS "ON")
set(VecGeom_NO_SPECIALIZATION "ON")
set(VecGeom_PLANESHELL "ON")
set(VecGeom_QUADRILATERAL_ACCELERATION "ON")
set(VecGeom_SINGLE_PRECISION "OFF")
set(VecGeom_USE_CACHED_TRANSFORMATIONS "OFF")
set(VecGeom_USE_INDEXEDNAVSTATES "ON")
set(VecGeom_USE_NAVINDEX "OFF")
set(VecGeom_VECTOR "empty")

# Hints for upstream dependencies
if(NOT DEFINED VecCore_DIR)
  set(VecCore_DIR "/usr/lib/arm-linux-gnueabihf/cmake/VecCore")
endif()
if(NOT DEFINED XercesC_INCLUDE_DIR)
  set(XercesC_INCLUDE_DIR "/usr/include")
endif()

# C++ Standard used in build
set(VecGeom_CXX_STANDARD "17")

# Define `COMPONENTS` argument to `find_package`
set(VecGeom_scalar_backend_FOUND True)
set(VecGeom_vc_backend_FOUND )
set(VecGeom_CUDA_FOUND ${VecGeom_ENABLE_CUDA})
set(VecGeom_GDML_FOUND ${VecGeom_GDML})
set(VecGeom_single_precision_FOUND ${VecGeom_SINGLE_PRECISION})

set(VecGeom_shared_FOUND ON)
if(VecGeom_static_FOUND)
  set(VecGeom_static_FOUND FALSE)
else()
  set(VecGeom_static_FOUND TRUE)
endif()

if(VecGeom_vc_FOUND)
  # Compatibility macro
  set(VecGeom_vector_backend_FOUND ON)
endif()

# Take default, but clear if it's "empty" as this is an internal "no flag supplied" case
set(VecGeom_VECTOR_ISA ${VecGeom_VECTOR})
if(VecGeom_VECTOR_ISA STREQUAL "empty")
  unset(VecGeom_VECTOR_ISA)
endif()

# For refinding required dependencies
include(CMakeFindDependencyMacro)

# Always VecCore
if(VecGeom_BUILTIN_VECCORE)
  # Must find our builtin where we expect
  find_dependency(VecCore 0.8.1 NO_DEFAULT_PATH COMPONENTS  )
else()
  find_dependency(VecCore 0.8.1 COMPONENTS )
endif()

# CUDA, if built
if(VecGeom_CUDA_FOUND)
  set(VecGeom_CUDA_STANDARD "")
  set(VecGeom_CUDA_ARCHITECTURES "")
  find_dependency(CUDAToolkit REQUIRED)
endif()

# GDML (Xerces-C), if built
if(VecGeom_GDML_FOUND)
  find_dependency(XercesC)
endif()

# DEPRECATED - ROOT only used for testing/benchmarking
set(VecGeom_ROOT_FOUND ${VecGeom_ROOT})
if(VecGeom_ROOT_FOUND)
  find_dependency(ROOT)
endif()

# DEPRECATED - Geant4 only used for testing/benchmarking
set(VecGeom_Geant4_FOUND ${VecGeom_GEANT4})
if(VecGeom_Geant4_FOUND)
  find_dependency(Geant4 )
endif()

# Include our target definitions
include("${CMAKE_CURRENT_LIST_DIR}/VecGeomTargets.cmake")

# Setup VECGEOM_LIBRARIES so that it should "work" for consumers
# _not_ implementing their own kernels/device code.
set(VECGEOM_LIBRARIES VecGeom::vecgeom)
if(TARGET VecGeom::vecgeomcuda)
  list(APPEND VECGEOM_LIBRARIES VecGeom::vecgeomcuda)
  set(VECGEOM_CUDA_STATIC_LIBRARY VecGeom::vecgeomcuda_static)
endif()

check_required_components(VecGeom)
